@@ -9,11 +9,11 @@ from sklearn.metrics import silhouette_score, silhouette_samples
 import numpy as np    
 
 
-from .clusters_utils import merge_tables, sort_by_leaves, make_dendrogram, make_colorbar_clusters, make_colorbar_metadata, make_legends, subset_k_best
+from gempipe.interface.clusters_utils import merge_tables, sort_by_leaves, make_dendrogram, make_colorbar_clusters, make_colorbar_metadata, make_legends, subset_k_best
 
     
     
-def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None, forcen=None, derive_report=None, report_key='species', legend_ratio=0.7, outfile=None, verbose=False):
+def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None, forcen=None, derive_report=None, report_key='species', legend_ratio=0.7, outfile=None, verbose=False, anchor=[None, None, None], key_to_color=None):
     """Perform a silhuette analysis to detect the optimal number of clusters. 
     
     Args:
@@ -32,7 +32,11 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
             Usually it is 'species' or 'niche'.
         legend_ratio (float): space reserved for the legend.
         outfile (str): filepath to be used to save the image. If `None` it will not be saved.
-        verbose (bool): if `True`, print more log messages
+        verbose (bool): if `True`, print more log messages.
+        anchor (list): list of tuples (X,Y) for customixing the position of legends. 
+            ``None`` will leave default positioning.
+        key_to_color (dict): dict mapping each category in `report_key` to a color in the format ([0:1],[0:1],[0:1]).
+            ``None`` will leave default color and order in the legend. 
     
     Returns:
         tuple: A tuple containing:
@@ -95,7 +99,7 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
             ax.fill_betweenx(np.arange(y_lower, y_upper),
                               0, cluster_i_scores,
                               facecolor=color, edgecolor=color, alpha=1.0)
-            ax.text(0, (y_lower + y_upper -1)/2, f'Cluster_{i+1}')
+            ax.text(0, (y_lower + y_upper -1)/2, f'Cluster_{i+1}', va='center', ha='left')
             y_lower = y_upper + -1  # no space between clusters
 
         ax.set_xlabel('Silhouette Coefficient')
@@ -105,7 +109,7 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
         ax.spines['top'].set_visible(False)
         ax.spines['right'].set_visible(False)
         ax.spines['left'].set_visible(False)
-        ax.set_title('Silhouette Plot for {} Clusters'.format(opt_n_clusters))
+        #ax.set_title('Silhouette Plot for {} Clusters'.format(opt_n_clusters))
         ax.set_facecolor('#f8f8f8')  # Light gray background
 
         return cluster_to_color
@@ -123,7 +127,7 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
         constant_columns = [col for col in data.columns if data[col].nunique() == 1]
         if verbose: print(f"WARNING: removing {len(constant_columns)} constant features.")
         data_bool = data_bool.drop(columns=constant_columns)
-    
+
     
     # pdist() / linkage() will loose the accession information. So here we save a dict: 
     index_to_acc = {i: accession for i, accession in enumerate(data_bool.index)}
@@ -190,7 +194,7 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
     # associated to a specific cluster. Thus, later we obtain the scores for a specific cluster 
     # simply with a 'silhouette_scores[clusters == i]'.
     
-
+    
     # Show silhouette scores for each datapoint (given the opimal number of clusters)
     cluster_to_color = make_plot_silhouette_coeff(axs[2], opt_n_clusters, silhouette_scores, clusters)
     
@@ -199,15 +203,15 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
     
     # order the dataframe following the leaves of the tree:
     ord_data_bool = sort_by_leaves(data_bool, linkage_matrix, index_to_acc)
-
+    
     # add colorbar for the dendrogram
     make_colorbar_clusters(axs[5], ord_data_bool, acc_to_cluster, cluster_to_color)
     
     # add colorbar for the species/niches
-    make_colorbar_metadata(axs[7], ord_data_bool, derive_report, report_key)
+    make_colorbar_metadata(axs[7], ord_data_bool, derive_report, report_key, key_to_color)
     
     # make legeneds
-    make_legends(axs[9], derive_report, report_key, cluster_to_color, dict_tables=None)
+    make_legends(axs[9], derive_report, report_key, cluster_to_color, None, anchor, key_to_color)
         
     # save to disk; bbox_inches='tight' removes white spaces around the figure. 
     if outfile != None:
@@ -218,7 +222,7 @@ def silhouette_analysis(tables, figsize = (10,5), drop_const=True, ctotest=None,
 
 
 
-def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=None, report_key='species', acc_to_cluster=None, cluster_to_color=None, legend_ratio=0.7, label_ratio=0.02, k=None, outfile=None, verbose=False):
+def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=None, report_key='species', acc_to_cluster=None, cluster_to_color=None, legend_ratio=0.7, label_ratio=0.02, k=None, outfile=None, verbose=False, anchor=[None, None, None], key_to_color=None):
     """Create a phylo-metabolic dendrogram.
     
     Args:
@@ -239,10 +243,15 @@ def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=
         k (int): focus on the `k` most discriminative features.
         outfile (str): filepath to be used to save the image. If `None` it will not be saved.
         verbose (bool): if `True`, print more log messages
+        anchor (list): list of tuples (X,Y) for customixing the position of legends. 
+            ``None`` will leave default positioning.
+        key_to_color (dict): dict mapping each category in `report_key` to a color in the format ([0:1],[0:1],[0:1]).
+            ``None`` will leave default color and order in the legend. 
     
     Returns:
         tuple: A tuple containing:
-            - matplotlib.figure.Figure: figure representing the sinhouette analysis.
+            - matplotlib.figure.Figure: figure representing the phylometabolic tree and associated heatmap.
+            - pnd.DataFrame: table representing the binary features contained in the heatmap.
     """
     
     
@@ -290,7 +299,7 @@ def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=
 
             # set X-axis labels: 
             ax.set_xticks(np.arange(ord_data.shape[1]))
-            ax.set_xticklabels(ord_data.columns, rotation=20, ha='right')
+            ax.set_xticklabels(ord_data.columns, rotation=30, ha='right')
             ax.xaxis.set_ticks_position('bottom')  # move ticks to the bottom
             ax.tick_params(top=False, bottom=True)  # disable ticks on the top
         
@@ -333,7 +342,8 @@ def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=
     make_dendrogram(axs[0], linkage_matrix)
 
     # order the dataframe following the leaves of the tree:
-    ord_data = sort_by_leaves(data, linkage_matrix, index_to_acc)   
+    ord_data = sort_by_leaves(data, linkage_matrix, index_to_acc)  
+    ord_data_bool = sort_by_leaves(data_bool, linkage_matrix, index_to_acc)  # only to return 
     
     # plot the heatmap:
     make_plot_heatmap_multilayer(axs[5], ord_data, k)
@@ -342,15 +352,141 @@ def heatmap_multilayer(tables, figsize = (10,5), drop_const=True, derive_report=
     make_colorbar_clusters(axs[1], ord_data, acc_to_cluster, cluster_to_color)
     
     # colorbar for the species/niche
-    make_colorbar_metadata(axs[3], ord_data, derive_report, report_key)
+    make_colorbar_metadata(axs[3], ord_data, derive_report, report_key, key_to_color)
     
     # make legends
-    make_legends(axs[7], derive_report, report_key, cluster_to_color, dict_tables)
+    make_legends(axs[7], derive_report, report_key, cluster_to_color, dict_tables, anchor, key_to_color)
     
     # save to disk; bbox_inches='tight' removes white spaces around the figure. 
     if outfile != None:
         plt.savefig(outfile, dpi=200, bbox_inches='tight')
                 
         
-    return pnd.concat([ord_data, derive_report], axis=1)
-    return fig
\ No newline at end of file
+    return fig, ord_data_bool
+
+
+
+def discriminant_feat(binary_feats, acc_to_cluster, cluster_to_color):
+    """Extract discriminant features from cluster of strains.
+    
+    Args:
+        tables (pnd.DataFrame): binary features table such as the one produced by `heatmap_multilayer`
+            (genomes in row, binary featuresin column).
+        acc_to_cluster (dict): dictionary such as the one produced by `silhouette_analysis``
+            (accessions as keys, cluster assignment as value).
+        cluster_to_color (dict): dictionary such as the one produced by `silhouette_analysis``
+            (clusters as keys, colors as value).
+        figsize (int, int): width and height of the figure. If ``None``, dimensions will automatically adapted.
+        
+    
+    Returns:
+        tuple: A tuple containing:
+            - matplotlib.figure.Figure: figure representing the discriminative binary features.
+    """
+    
+    def get_contingency(binary_feats, feat_id):
+        contingency_table = pnd.crosstab(
+            binary_feats[feat_id],
+            binary_feats['y'],
+            margins = False)
+        
+        # the resulting pnd.DataFrame will be similar to (e.g. for the binary feat "[aux]his__L"):
+        #         y            Cluster_1  Cluster_2  Cluster_3  Cluster_4  Cluster_5
+        # [aux]his__L                                                       
+        # 0                   12          3          6         12          0
+        # 1                    0          0          0          0          3
+        return contingency_table
+    
+    
+    # START
+    # convert to int:
+    binary_feats = binary_feats.copy().astype(int)  # .copy() will defragment the dataframe.
+    
+    
+    # add the classification column (usually caled 'y'):
+    binary_feats['y'] = "Cluster_" + str(0)
+    for accession, row in binary_feats.iterrows(): 
+        binary_feats.loc[accession, 'y'] = "Cluster_" + str(acc_to_cluster[accession]) # str to avoid ambiguity
+        
+    
+    # get dataframe of relative frequencies:
+    df_relfreq = pnd.DataFrame(index=list(set(list(binary_feats.columns))-set(['y'])), columns=binary_feats['y'].unique())
+    for feat_id in df_relfreq.index:
+        cont = get_contingency(binary_feats, feat_id)
+        for cluster in df_relfreq.columns:
+            df_relfreq.loc[feat_id, cluster] = cont.loc[1, cluster] / (cont.loc[1, cluster] + cont.loc[0, cluster])
+    df_relfreq  = df_relfreq.astype(float)
+    
+    
+    # filter the dataframe:
+    df_relfreq = df_relfreq[(df_relfreq >= 0.90).any(axis=1)]
+    df_relfreq = df_relfreq[(df_relfreq <= 0.10).any(axis=1)]
+        
+        
+    # invert column order to match that of the heatmap
+    df_relfreq = df_relfreq[reversed(df_relfreq.columns)]
+    
+    
+    # sort features aalphabetically
+    df_relfreq = df_relfreq.sort_index(ascending=False)
+    
+    # resort index in order to have similar rows (similar features) close together.
+    index_to_featid = {i: feat_id for i, feat_id in enumerate(df_relfreq.index)}
+    distances = pdist(df_relfreq, 'jaccard')
+    linkage_matrix = linkage(distances, method='ward')
+    df_relfreq = sort_by_leaves(df_relfreq, linkage_matrix, index_to_featid)  # only to return 
+    
+    
+    # create the subplots: 
+    fig, axs = plt.subplots(
+        nrows=2, ncols=1, 
+        figsize=(0.5 * len(df_relfreq.columns), 0.3 * len(df_relfreq)), # global dimensions.
+        gridspec_kw={'width_ratios': [1], 'height_ratios': [1, 1*len(df_relfreq)]}) # suplots width proportions. 
+    # adjust the space between subplots: 
+    plt.subplots_adjust(wspace=0, hspace=0)
+    axs[0].set_frame_on(False)  # remove squared border but not ticks
+    axs[1].set_frame_on(False)  # remove squared border but not ticks
+    
+    
+    # create matshow (clusters)
+    cmap = LinearSegmentedColormap.from_list('', [cluster_to_color[eval(cluster.replace('Cluster_', ''))] for cluster in df_relfreq.columns])
+    df_clusters = pnd.DataFrame({cluster: [i] for i, cluster in enumerate(df_relfreq.columns)})
+    axs[0].matshow(
+        df_clusters,  
+        cmap=cmap,
+        vmin=df_clusters.min().min(), vmax=df_clusters.max().max(), # define ranges for the colormap.
+        aspect='auto', # fixed axes and aspect adjusted to fit data.
+        interpolation='none') # no interp. performed on Agg-ps-pdf-svg backends.
+    
+    
+    # create matshow (heatmap)
+    cmap = LinearSegmentedColormap.from_list('', ["#DDDDDD", "#8888DD"])
+    axs[1].matshow(
+        df_relfreq,  
+        cmap=cmap,
+        vmin=df_relfreq.min().min(), vmax=df_relfreq.max().max(), # define ranges for the colormap.
+        aspect='auto', # fixed axes and aspect adjusted to fit data.
+        interpolation='none') # no interp. performed on Agg-ps-pdf-svg backends.
+    
+    
+    # add x/y axis labels (clusters):
+    axs[0].set_xticks(range(len(df_relfreq.columns)))  # Position of x-ticks
+    axs[0].set_xticklabels(df_relfreq.columns, ha='left', rotation=30)  # Replace with your desired labels
+    axs[0].set_yticks([])   # remove x ticks
+    
+    
+    # add x/y axis labels (heatmap):
+    axs[1].set_xticks([])   # remove x ticks
+    axs[1].set_yticks(range(len(df_relfreq.index)))  # Position of y-ticks
+    axs[1].set_yticklabels(df_relfreq.index)
+    
+    
+    # add annotations (rel frequencies):
+    for i in range(len(df_relfreq.index)):
+        for j in range(len(df_relfreq.columns)):
+            freq_to_show = round(df_relfreq.iloc[i,j], 2)
+            annot_color = 'white' if freq_to_show >= 0.60 else 'black'
+            axs[1].text(j, i, f'{freq_to_show}', ha='center', va='center', color=annot_color)
+    
+
+    return fig, df_relfreq
\ No newline at end of file
